@@ -29,7 +29,7 @@ class PageViewMessage : public QWidget
     public:
 		PageViewMessage( QWidget * parent );
 
-		enum Icon { None, Info, Warning, Error, Find };
+		enum Icon { None = 0, Info, Warning, Error, Find };
 		void display( const QString & message, Icon icon = Info, int durationMs = 4000 );
     public slots:
 	void showText( const QString &);
