@@ -14,12 +14,18 @@
 #include "qtermconfig.h"
 #include "qtermparam.h"
 #include "qtermconvert.h"
+#ifdef KWALLET_ENABLED
+#include "wallet.h"
+#endif // KWALLET_ENABLED
 #include "qterm.h"
 #include <QtCore/QDir>
 #include <QtCore/QFileInfo>
 #include <QtCore/QTranslator>
 #include <QtCore/QVariant>
+#include <QtCore/QUrl>
+#include <QtCore/QProcess>
 #include <QtGui/QApplication>
+#include <QtGui/QDesktopServices>
 #include <QtGui/QFileDialog>
 #include <QtGui/QMessageBox>
 
@@ -61,6 +67,14 @@ Global::Global()
     m_config = new Config(m_fileCfg);
     m_address = new Config(m_addrCfg);
     m_converter = new Convert();
+#ifdef KWALLET_ENABLED
+    if (Wallet::isWalletAvailable()) {
+        qDebug() << "KWallet service found";
+        m_wallet = new Wallet(this);
+    } else {
+        m_wallet = NULL;
+    }
+#endif // KWALLET_ENABLED
     if (!iniSettings()) {
         m_status = INIT_ERROR;
         return;
@@ -131,10 +145,13 @@ QStringList Global::loadNameList()
 bool Global::loadAddress(int n, Param& param)
 {
     QString strTmp, strSection;
-    if (n < 0)
+    strTmp = m_address->getItemValue("bbs list", "num").toString();
+    if ((n < 0 && strTmp.toInt() <= 0) || n < -1)
         strSection = "default";
-    else
+    else {
+        n = n < 0 ? 0 : n;
         strSection.sprintf("bbs %d", n);
+    }
 
     // check if larger than existence
     strTmp = m_address->getItemValue("bbs list", "num").toString();
@@ -150,7 +167,13 @@ bool Global::loadAddress(int n, Param& param)
     param.m_bAutoLogin = (strTmp != "0");
     param.m_strPreLogin = m_address->getItemValue(strSection, "prelogin").toString();
     param.m_strUser = m_address->getItemValue(strSection, "user").toString();
-    param.m_strPasswd = m_address->getItemValue(strSection, "password").toString();
+#ifdef KWALLET_ENABLED
+    if (m_wallet != NULL) {
+        m_wallet->open();
+        param.m_strPasswd = m_wallet->readPassword(param.m_strName, param.m_strUser);
+    } else
+#endif // KWALLET_ENABLED
+        param.m_strPasswd = m_address->getItemValue(strSection, "password").toString();
     param.m_strPostLogin = m_address->getItemValue(strSection, "postlogin").toString();
 
     strTmp = m_address->getItemValue(strSection, "bbscode").toString();
@@ -163,12 +186,11 @@ bool Global::loadAddress(int n, Param& param)
     param.m_bAlwaysHighlight = (strTmp != "0");
     strTmp = m_address->getItemValue(strSection, "ansicolor").toString();
     param.m_bAnsiColor = (strTmp != "0");
-    param.m_strFontName = m_address->getItemValue(strSection, "fontname").toString();
+    param.m_strASCIIFontName = m_address->getItemValue(strSection, "asciifont").toString();
+    param.m_strGeneralFontName = m_address->getItemValue(strSection, "generalfont").toString();
     strTmp = m_address->getItemValue(strSection, "fontsize").toString();
     param.m_nFontSize = strTmp.toInt();
-    param.m_clrFg.setNamedColor(m_address->getItemValue(strSection, "fgcolor").toString());
-    param.m_clrBg.setNamedColor(m_address->getItemValue(strSection, "bgcolor").toString());
-    param.m_strSchemaFile = m_address->getItemValue(strSection, "schemafile").toString();
+    param.m_strSchemeFile = m_address->getItemValue(strSection, "schemefile").toString();
 
     param.m_strTerm = m_address->getItemValue(strSection, "termtype").toString();
     strTmp = m_address->getItemValue(strSection, "keytype").toString();
@@ -216,6 +238,9 @@ bool Global::loadAddress(int n, Param& param)
     strTmp = m_address->getItemValue(strSection, "loadscript").toString();
     param.m_bLoadScript = (strTmp != "0");
     param.m_strScriptFile = m_address->getItemValue(strSection, "scriptfile").toString();
+    if (param.m_strScriptFile.isEmpty()) {
+        param.m_bLoadScript = false;
+    }
 
     strTmp = m_address->getItemValue(strSection, "menutype").toString();
     param.m_nMenuType = strTmp.toInt();
@@ -241,7 +266,14 @@ void Global::saveAddress(int n, const Param& param)
     m_address->setItemValue(strSection, "autologin", param.m_bAutoLogin ? "1" : "0");
     m_address->setItemValue(strSection, "prelogin", param.m_strPreLogin);
     m_address->setItemValue(strSection, "user", param.m_strUser);
-    m_address->setItemValue(strSection, "password", param.m_strPasswd);
+
+#ifdef KWALLET_ENABLED
+    if (m_wallet != NULL) {
+        m_wallet->open();
+        m_wallet->writePassword(param.m_strName, param.m_strUser, param.m_strPasswd);
+    } else
+#endif // KWALLET_ENABLED
+        m_address->setItemValue(strSection, "password", param.m_strPasswd);
     m_address->setItemValue(strSection, "postlogin", param.m_strPostLogin);
 
     strTmp=param.m_BBSCode;
@@ -251,12 +283,11 @@ void Global::saveAddress(int n, const Param& param)
     m_address->setItemValue(strSection, "autofont", param.m_bAutoFont ? "1" : "0");
     m_address->setItemValue(strSection, "alwayshighlight", param.m_bAlwaysHighlight ? "1" : "0");
     m_address->setItemValue(strSection, "ansicolor", param.m_bAnsiColor ? "1" : "0");
-    m_address->setItemValue(strSection, "fontname", param.m_strFontName);
+    m_address->setItemValue(strSection, "asciifont", param.m_strASCIIFontName);
+    m_address->setItemValue(strSection, "generalfont", param.m_strGeneralFontName);
     strTmp.setNum(param.m_nFontSize);
     m_address->setItemValue(strSection, "fontsize", strTmp);
-    m_address->setItemValue(strSection, "fgcolor", param.m_clrFg.name());
-    m_address->setItemValue(strSection, "bgcolor", param.m_clrBg.name());
-    m_address->setItemValue(strSection, "schemafile", param.m_strSchemaFile);
+    m_address->setItemValue(strSection, "schemefile", param.m_strSchemeFile);
 
     m_address->setItemValue(strSection, "termtype", param.m_strTerm);
     strTmp.setNum(param.m_nKey);
@@ -304,6 +335,26 @@ void Global::saveAddress(int n, const Param& param)
 
 }
 
+void Global::removeAddress(int n)
+{
+    if (n < 0)
+        return;
+    QString strSection = QString("bbs %1").arg(n);
+#ifdef KWALLET_ENABLED
+    // check if larger than existence
+    QString strTmp = m_address->getItemValue("bbs list", "num").toString();
+    if (n >= strTmp.toInt())
+        return;
+    QString site = m_address->getItemValue(strSection, "name").toString();
+    QString username = m_address->getItemValue(strSection, "user").toString();
+    if (m_wallet != NULL) {
+        m_wallet->open();
+        m_wallet->removePassword(site, username);
+    }
+#endif // KWALLET_ENABLED
+    m_address->deleteSection(strSection);
+}
+
 void Global::loadPrefence()
 {
     QString strTmp;
@@ -367,16 +418,16 @@ QString Global::getSaveFileName(const QString& filename, QWidget* widget)
     // get the previous dir
     QString path = m_config->getItemValue("global", "savefiledialog").toString();
 
-    QString strSave = QFileDialog::getSaveFileName(widget, "Choose a file to save under", path + "/" + filename, "*");
+    QString strSave = QFileDialog::getSaveFileName(widget, tr("Choose a file to save under"), path + "/" + filename, "*");
 
     QFileInfo fi(strSave);
 
     while (fi.exists()) {
         int yn = QMessageBox::warning(widget, "QTerm",
-                                      "File exists. Overwrite?", "Yes", "No");
+                                      tr("File exists. Overwrite?"), tr("Yes"), tr("No"));
         if (yn == 0)
             break;
-        strSave = QFileDialog::getSaveFileName(widget, "Choose a file to save under", path + "/" + filename, "*");
+        strSave = QFileDialog::getSaveFileName(widget, tr("Choose a file to save under"), path + "/" + filename, "*");
         if (strSave.isEmpty())
             break;
     }
@@ -418,8 +469,8 @@ bool Global::iniWorkingDir(QString param)
     strcat(_addrCfg, "address.cfg");
     m_addrCfg = QString::fromLocal8Bit(_addrCfg);
 
-    QString pathSchema = m_pathCfg + "schema";
-    if (!isPathExist(pathSchema))
+    QString pathScheme = m_pathCfg + "scheme";
+    if (!isPathExist(pathScheme))
         return false;
     return true;
 }
@@ -439,13 +490,15 @@ bool Global::iniWorkingDir(QString param)
     fi.setFile(param);
     m_pathLib = fi.path() + '/';
 #else
+    QString prefix = QCoreApplication::applicationDirPath();
+    prefix.chop(3); // "bin"
     m_pathCfg = QDir::homePath() + "/.qterm/";
     if (!isPathExist(m_pathCfg))
         return false;
 
     // pathLib --- where datedir "pic", "cursor", "po"
     if (param.indexOf('/') == -1)
-        m_pathLib = QTERM_DATADIR"/";
+        m_pathLib = prefix + "share/qterm/";
     else {
         QFileInfo fi(param);
         if (fi.isSymLink())
@@ -454,9 +507,9 @@ bool Global::iniWorkingDir(QString param)
         param.truncate(param.lastIndexOf('/'));
         QString oldPath = QDir::currentPath();
         QDir::setCurrent(param);
-        dir.setPath(QTERM_BINDIR);
+        dir.setPath(QCoreApplication::applicationDirPath());
         if (dir == QDir::current())
-            m_pathLib = QTERM_DATADIR;
+            m_pathLib = prefix + "share/qterm/";
         else
             m_pathLib = QDir::currentPath();
         QDir::setCurrent(oldPath);
@@ -464,8 +517,8 @@ bool Global::iniWorkingDir(QString param)
     }
 #endif
 
-    QString pathSchema = m_pathCfg + "schema";
-    if (!isPathExist(pathSchema))
+    QString pathScheme = m_pathCfg + "scheme";
+    if (!isPathExist(pathScheme))
         return false;
 
 
@@ -493,6 +546,16 @@ bool Global::iniSettings()
 {
     //install the translator
     QString lang = m_config->getItemValue("global", "language").toString();
+    if (lang == "eng")
+        m_language = Global::English;
+    else if (lang == "chs")
+        m_language = Global::SimpilifiedChinese;
+    else if (lang == "cht")
+        m_language = Global::TraditionalChinese;
+    else {
+        qDebug("Language setting is not correct");
+        m_language = Global::English;
+    }
     if (lang != "eng" && !lang.isEmpty()) {
         // look in $HOME/.qterm/po/ first
         QString qm = QDir::homePath() + "/.qterm/po/qterm_" + lang + ".qm";
@@ -795,6 +858,10 @@ void Global::saveState(const QByteArray state)
 
 void Global::cleanup()
 {
+#ifdef KWALLET_ENABLED
+    if (m_wallet != NULL)
+        m_wallet->close();
+#endif // KWALLET_ENABLED
     if (m_pref.bClearPool) {
         clearDir(m_pref.strZmPath);
         clearDir(m_pref.strPoolPath);
@@ -802,23 +869,31 @@ void Global::cleanup()
     }
 }
 
-void Global::openUrl(const QString & url)
+void Global::openUrl(const QString & urlStr)
 {
     QString command = m_pref.strHttp;
+    if (command.isEmpty()) {
+        QUrl url(urlStr,QUrl::TolerantMode);
+        if (!QDesktopServices::openUrl(url)) {
+            qDebug("Failed to open the url with QDesktopServices");
+        }
+        return;
+    }
+
     if(command.indexOf("%L")==-1) // no replace
         //QApplication::clipboard()->setText(strUrl);
-        command += " \"" + url +"\"";
+        command += " \"" + urlStr +"\"";
     else
-        command.replace("%L",  "\"" + url + "\"");
+        command.replace("%L",  "\"" + urlStr + "\"");
         //cstrCmd.replace("%L",  strUrl.toLocal8Bit());
 
-    //qDebug()<<"run command " << strCmd;
-    //QProcess::startDetached(strCmd);
-    //TODO: How to do this in Windows?
 #if !defined(_OS_WIN32_) && !defined(Q_OS_WIN32)
     command += " &";
-#endif
     system(command.toUtf8().data());
+#else
+    QProcess::startDetached(command);
+#endif
+
 }
 
 QString Global::convert(const QString & source, Global::Conversion flag)
