@@ -22,6 +22,7 @@ AUTHOR:        kingson fiasco
 #include "qtermtelnet.h"
 #include "qtermconfig.h"
 #include "qtermglobal.h"
+#include "schemedialog.h"
 
 #include <QApplication>
 #include <QPainter>
@@ -64,12 +65,15 @@ Screen::Screen(QWidget *parent, Buffer *buffer, Param *param, BBS *bbs)
     //m_pCanvas = NULL;
     m_ePaintState = Repaint;
     m_bCursor = true;
+    m_inputContent = NULL;
+    m_pASCIIFont = NULL;
+    m_pGeneralFont = NULL;
 
     setFocusPolicy(Qt::ClickFocus);
     setAttribute(Qt::WA_InputMethodEnabled, true);
     setMouseTracking(true);
 
-    setSchema();
+    setScheme();
 
     initFontMetrics();
 
@@ -115,7 +119,7 @@ Screen::~Screen()
     delete m_blinkTimer;
     delete m_cursorTimer;
     //delete m_pCanvas;
-    //delete m_inputContent;
+    delete m_inputContent;
 }
 
 // focus event received
@@ -292,44 +296,22 @@ void Screen::wheelEvent(QWheelEvent * we)
 
 void Screen::initFontMetrics()
 {
-    if (m_pParam->m_bAutoFont)
-        m_pFont = new QFont(m_pParam->m_strFontName);
-    else {
-        m_pFont = new QFont(m_pParam->m_strFontName, qMax(8, m_pParam->m_nFontSize));
-        QFontMetrics *fm = new QFontMetrics(*m_pFont);
-
-        getFontMetrics(fm);
-        delete fm;
+    if (m_pASCIIFont != NULL) {
+        delete m_pASCIIFont;
     }
-    m_pFont->setWeight(QFont::Normal);
-
-    m_pFont->setStyleStrategy(Global::instance()->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
-}
-
-void Screen::setDispFont(const QFont& font)
-{
-    delete m_pFont;
-    if (m_pParam->m_bAutoFont)
-        m_pFont = new QFont(font.family());
-    else {
-        m_pFont = new QFont(font);
-
-        QFontInfo fi(*m_pFont);
-        //int nSize = fi.pixelSize();
-        QFontMetrics *fm = new QFontMetrics(*m_pFont);
-        getFontMetrics(fm);
-        delete fm;
+    if (m_pGeneralFont != NULL) {
+        delete m_pGeneralFont;
     }
+    m_pASCIIFont = new QFont(m_pParam->m_strASCIIFontName, qMax(8, m_pParam->m_nFontSize));
+    m_pGeneralFont = new QFont(m_pParam->m_strGeneralFontName, qMax(8, m_pParam->m_nFontSize));
 
-    m_pFont->setStyleStrategy(Global::instance()->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
+    m_pASCIIFont->setWeight(QFont::Normal);
+    m_pGeneralFont->setWeight(QFont::Normal);
+
+    m_pASCIIFont->setStyleStrategy(Global::instance()->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
+    m_pGeneralFont->setStyleStrategy(Global::instance()->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
 }
 
-QFont Screen::getDispFont()
-{
-    return *m_pFont;
-}
-
-
 void Screen::updateFont()
 {
     int nPixelSize;
@@ -337,18 +319,18 @@ void Screen::updateFont()
                                 m_rcClient.width() * 2 / m_pBuffer->columns()));
 
     for (nPixelSize = nIniSize - 3; nPixelSize <= nIniSize + 3; nPixelSize++) {
-        m_pFont->setPixelSize(nPixelSize);
+        m_pASCIIFont->setPixelSize(nPixelSize);
+        m_pGeneralFont->setPixelSize(nPixelSize);
 
-        QFontMetrics fm(*m_pFont);
-        getFontMetrics(&fm);
+        getFontMetrics();
         if ((m_pBuffer->line()*m_nCharHeight) > m_rcClient.height()
                 || (m_pBuffer->columns()*m_nCharWidth) > m_rcClient.width()) {
             while (nPixelSize > 5) {
                 nPixelSize--;
-                m_pFont->setPixelSize(nPixelSize);
+                m_pASCIIFont->setPixelSize(nPixelSize);
+                m_pGeneralFont->setPixelSize(nPixelSize);
 
-                QFontMetrics fm2(*m_pFont);
-                getFontMetrics(&fm2);
+                getFontMetrics();
                 if ((m_pBuffer->line()*m_nCharHeight) < m_rcClient.height()
                         && (m_pBuffer->columns()*m_nCharWidth) < m_rcClient.width())
                     break;
@@ -356,26 +338,88 @@ void Screen::updateFont()
             break;
         }
     }
-    int marginw = (m_rcClient.width() - (m_pBuffer->columns()*m_nCharWidth))/2;
-    int marginh = (m_rcClient.height() - (m_pBuffer->line()*m_nCharHeight))/2;
+    int marginw,marginh;
+    marginw=marginh=0;
+    if (Frame::instance()->isMaximized()||Frame::instance()->isFullScreen()) {
+        marginw = (m_rcClient.width() - (m_pBuffer->columns()*m_nCharWidth))/2;
+        marginh = (m_rcClient.height() - (m_pBuffer->line()*m_nCharHeight))/2;
+    }
     QPoint point = m_rcClient.topLeft();
     m_rcClient = QRect(point.x()+marginw, point.y()+marginh, m_pBuffer->columns()*m_nCharWidth, m_pBuffer->line()*m_nCharHeight);
-    m_pFont->setWeight(QFont::Normal);
-    m_pFont->setStyleStrategy(Global::instance()->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
+    m_pASCIIFont->setWeight(QFont::Normal);
+    m_pGeneralFont->setWeight(QFont::Normal);
+    m_pASCIIFont->setStyleStrategy(Global::instance()->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
+    m_pGeneralFont->setStyleStrategy(Global::instance()->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
+    if (m_inputContent != NULL) {
+        delete m_inputContent;
+        m_inputContent = NULL;
+    }
 }
 
-void Screen::getFontMetrics(QFontMetrics *fm)
+void Screen::getFontMetrics()
 {
-    float cn = fm->width(QChar(0x4e00));
-    float en = fm->width('W');
+    QFontMetrics ascii_fm(*m_pASCIIFont);
+    QFontMetrics general_fm(*m_pGeneralFont);
+    float cn = general_fm.width(QChar(0x4e00));
+    float en = ascii_fm.width('W');
     if (en / cn < 0.7) // almost half
         m_nCharWidth = qMax((cn + 1) / 2, en);
     else
         m_nCharWidth = (qMax(en, cn) + 1) / 2;
 
-    m_nCharHeight = fm->height();
-    m_nCharAscent = fm->ascent();
-    m_nCharDescent = fm->descent();
+    m_nCharDelta = m_nCharWidth - cn/2;
+    m_nCharHeight = ascii_fm.height();
+    m_nCharAscent = ascii_fm.ascent();
+    m_nCharDescent = ascii_fm.descent();
+}
+
+void Screen::asciiFontChanged(const QFont & font)
+{
+    qDebug() << "ASCII font changed: " << font.family();
+    if (m_pASCIIFont != NULL) {
+        delete m_pASCIIFont;
+    }
+    m_pASCIIFont = new QFont(font);
+    m_pASCIIFont->setPixelSize(qMax(8,m_pParam->m_nFontSize));
+    m_pGeneralFont->setPixelSize(qMax(8,m_pParam->m_nFontSize));
+    QResizeEvent* re = new QResizeEvent(size(), size());
+    resizeEvent(re);
+}
+
+void Screen::generalFontChanged(const QFont & font)
+{
+    if (m_pGeneralFont != NULL) {
+        delete m_pGeneralFont;
+    }
+    m_pGeneralFont = new QFont(font);
+    m_pASCIIFont->setPixelSize(qMax(8,m_pParam->m_nFontSize));
+    m_pGeneralFont->setPixelSize(qMax(8,m_pParam->m_nFontSize));
+    QResizeEvent* re = new QResizeEvent(size(), size());
+    resizeEvent(re);
+}
+
+void Screen::fontSizeChanged(int value)
+{
+    m_pParam->m_nFontSize = value;
+    m_pASCIIFont->setPixelSize(qMax(8,m_pParam->m_nFontSize));
+    m_pGeneralFont->setPixelSize(qMax(8,m_pParam->m_nFontSize));
+    QResizeEvent* re = new QResizeEvent(size(), size());
+    resizeEvent(re);
+}
+
+QFont Screen::asciiFont()
+{
+    return *m_pASCIIFont;
+}
+
+QFont Screen::generalFont()
+{
+    return *m_pGeneralFont;
+}
+
+int Screen::fontSize()
+{
+    return m_pParam->m_nFontSize;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -383,7 +427,7 @@ void Screen::getFontMetrics(QFontMetrics *fm)
 /*                                 Colors                                   */
 /*                                                                         */
 /* ------------------------------------------------------------------------ */
-void Screen::setSchema()
+void Screen::setScheme()
 {
 // the default color table
     m_color[0]  = Qt::black;
@@ -407,11 +451,11 @@ void Screen::setSchema()
 
 
 
-// if we have schema defined
-    if (QFile::exists(m_pParam->m_strSchemaFile)) {
+// if we have scheme defined
+    if (QFile::exists(m_pParam->m_strSchemeFile)) {
 
-//  printf("schema %s loaded sucessfully\n", m_pParam->m_strSchemaFile);
-        Config *pConf = new Config(m_pParam->m_strSchemaFile);
+//  printf("scheme %s loaded sucessfully\n", m_pParam->m_strSchemeFile);
+        Config *pConf = new Config(m_pParam->m_strSchemeFile);
 
         m_color[0].setNamedColor(pConf->getItemValue("color", "color0").toString());
         m_color[1].setNamedColor(pConf->getItemValue("color", "color1").toString());
@@ -430,30 +474,20 @@ void Screen::setSchema()
         m_color[14].setNamedColor(pConf->getItemValue("color", "color14").toString());
         m_color[15].setNamedColor(pConf->getItemValue("color", "color15").toString());
 
-        // bg type
-        QString strTmp = pConf->getItemValue("image", "type").toString();
-        m_nPxmType = strTmp.toInt();
-
-        // fade effect
-        QColor fadecolor;
-        fadecolor.setNamedColor(pConf->getItemValue("image", "fade").toString());
-        strTmp = pConf->getItemValue("image", "alpha").toString();
-        float alpha = strTmp.toFloat();
-
-        // get the image name
-        if (QFile::exists(pConf->getItemValue("image", "name").toString()) && m_nPxmType > 1) { // valid image name and type
-            m_pxmBg = QPixmap(pConf->getItemValue("image", "name").toString());
-            QImage ima(m_pxmBg.toImage());
-            ima = fade(ima, alpha, fadecolor);
-            m_pxmBg = QPixmap::fromImage(ima);
-
-        }
         delete pConf;
     }
+}
 
-// override schema using user defined Fg/Bg color
-    m_color[0]  = m_pParam->m_clrBg;
-    m_color[7]  = m_pParam->m_clrFg;
+void Screen::schemeChanged(int index)
+{
+    if (index == -1) {
+        return;
+    }
+    QStringList schemeList = schemeDialog::loadSchemeList();
+    m_pParam->m_strSchemeFile = schemeList[index];
+    setScheme();
+    QResizeEvent* re = new QResizeEvent(size(), size());
+    resizeEvent(re);
 }
 
 
@@ -681,6 +715,7 @@ void Screen::blinkScreen()
                     startx = i;
                     while (i < linelength && GETBLINK(attr.at(i)))
                         ++i;
+                    --i;
                     drawLine(painter, index, startx, i, false);
                 }
             }
@@ -806,14 +841,13 @@ void Screen::repaintScreen(QPaintEvent * pe)
 
 //  qDebug("rcClient: %d, %d", m_pCanvas->width(),m_pCanvas->height());
 
+    painter.eraseRect(pe->rect());
+
     QRect rect = pe->rect() & m_rcClient;
-
-    if (rect.isEmpty()) {
-        painter.eraseRect(pe->rect());
+    if (rect.isEmpty())
         return;
-    }
 
-    painter.eraseRect(rect);
+    //painter.eraseRect(rect);
 
     QPoint tlPoint = mapToChar(QPoint(rect.left(), rect.top()));
 
@@ -846,7 +880,8 @@ void Screen::drawLine(QPainter& painter, int index, int beginx, int endx, bool c
     QByteArray color = pTextLine->getColor();
     QByteArray attr = pTextLine->getAttr();
     int linelength = pTextLine->getLength();
-    char tempcp, tempea;
+    char tempcp = 0;
+    char tempea = 0;
     short tempattr;
     bool bSelected;
     bool bReverse = false;
@@ -857,8 +892,13 @@ void Screen::drawLine(QPainter& painter, int index, int beginx, int endx, bool c
     if (beginx < 0)
         beginx = 0;
 
-    if (endx >= linelength || endx < 0)
-        endx = linelength-1;
+    if (endx >= linelength || endx < 0) {
+        endx = qMin(m_pBuffer->columns(), linelength)-1;
+        painter.eraseRect(mapToRect(beginx, index, linelength, 1)); // Maybe we should calculate more accurate size;
+    }
+    if (endx >= qMin(color.size(), attr.size())) {
+        endx = qMin(color.size(), attr.size()) -1;
+    }
 
     if (beginx >= linelength) {
         //qDebug("Screen::drawLine: wrong position: %d(%d)", beginx, linelength);
@@ -890,15 +930,13 @@ void Screen::drawLine(QPainter& painter, int index, int beginx, int endx, bool c
             tempea = attr.at(i);
         bSelected = m_pBuffer->isSelected(QPoint(i, index), m_pWindow->m_bCopyRect);
         len = pTextLine->size(i);
+        if ( (i+1) >= linelength) {
+            len = 1;
+        }
         if (len <= 0) {
             qDebug("drawLine: non printable char");
             continue;
         }
-        if (tempcp != color.at(i+len-1) || tempea != attr.at(i+len-1) || bSelected != m_pBuffer->isSelected(QPoint(i+len-1, index), m_pWindow->m_bCopyRect)) {
-            //TODO: draw half char
-            qDebug("drawLine: not implemented yet");
-        }
-        // get str of the same attribute
 
         if (bSelected) // selected area is text=color(0) background=color(7)
             tempattr = SETCOLOR(SETFG(0) | SETBG(7)) | SETATTR(NO_ATTR);
@@ -912,22 +950,41 @@ void Screen::drawLine(QPainter& painter, int index, int beginx, int endx, bool c
         // TODO: Rewrite this when we want to do more than char to char convert
         strShow = Global::instance()->convert(pTextLine->getText(startx, len), (Global::Conversion)m_pParam->m_nDispCode);
 
-
+        if (strShow.isEmpty()) {
+            qDebug("drawLine: empty string?");
+            continue;
+        }
         // Draw Charactors one by one to fix the variable font display problem
         int charWidth = TermString::wcwidth(strShow.at(0));
         if (charWidth <= 0) {
             qDebug("drawLine: non printable char");
             continue;
+        } else if (charWidth == 1) {
+            painter.setFont(*m_pASCIIFont);
+        } else if (charWidth == 2) {
+            painter.setFont(*m_pGeneralFont);
         }
-        drawStr(painter, (QString)strShow.at(0), startx, index, charWidth, tempattr, bSelected);
+        CharFlags flags = RenderAll;
+        if ( pTextLine->isPartial(startx) ) {
+            flags = RenderRight;
+            charWidth = 1;
+        } else if ( charWidth == 2) {
+            if (tempcp != color.at(i+1) || tempea != attr.at(i+1) || bSelected != m_pBuffer->isSelected(QPoint(i+1, index), m_pWindow->m_bCopyRect)) {
+                charWidth = 1;
+                flags = RenderLeft;
+            } else {
+                ++i;
+            }
+        }
+        drawStr(painter, (QString)strShow.at(0), startx, index, charWidth, tempattr, bSelected, flags);
 
-        i += (len-1);
+        //i += (len-1);
     }
 }
 
 // draw functions
 void Screen::drawStr(QPainter& painter, const QString& str, int x, int y, int length,
-                     short attribute, bool transparent)
+                     short attribute, bool transparent, CharFlags flags)
 {
     char cp = GETCOLOR(attribute);
     char ea = GETATTR(attribute);
@@ -940,11 +997,11 @@ void Screen::drawStr(QPainter& painter, const QString& str, int x, int y, int le
     };
     // test underline mask
     if (GETUNDERLINE(ea)) {
-        m_pFont->setUnderline(true);
-        painter.setFont(*m_pFont);
+        m_pASCIIFont->setUnderline(true);
+        m_pGeneralFont->setUnderline(true);
     } else {
-        m_pFont->setUnderline(false);
-        painter.setFont(*m_pFont);
+        m_pASCIIFont->setUnderline(false);
+        m_pGeneralFont->setUnderline(false);
     }
     // test blink mask
     if (GETBLINK(ea)) {
@@ -970,7 +1027,7 @@ void Screen::drawStr(QPainter& painter, const QString& str, int x, int y, int le
         else
             painter.setBackgroundMode(Qt::TransparentMode);
         painter.setBackground(GETBG(cp) == 7 ? Qt::black : Qt::white);
-        painter.drawText(pt.x(), pt.y() + m_nCharAscent, str);
+        painter.drawText(pt.x(), pt.y(), m_nCharWidth*length, m_nCharHeight, flags, str);
         return;
     }
 
@@ -992,8 +1049,16 @@ void Screen::drawStr(QPainter& painter, const QString& str, int x, int y, int le
     } else {
         if (GETBG(cp) != 0 || m_ePaintState == Cursor)
             painter.fillRect(mapToRect(x, y, length, 1), QBrush(m_color[GETBG(cp)]));
-        painter.drawText(pt.x(), pt.y() + m_nCharAscent, str);
+        if (flags == RenderAll) {
+            painter.drawText(pt.x()+m_nCharDelta, pt.y(), m_nCharWidth*length, m_nCharHeight, Qt::AlignLeft|Qt::AlignBottom, str);
+        } else if (flags == RenderLeft) {
+            painter.drawText(pt.x()+m_nCharDelta, pt.y(), m_nCharWidth-m_nCharDelta, m_nCharHeight, Qt::AlignLeft|Qt::AlignBottom, str);
+        } else if (flags == RenderRight) {
+            int width = painter.fontMetrics().width(str[0])-m_nCharWidth+m_nCharDelta;
+            painter.drawText(pt.x(), pt.y(), width, m_nCharHeight, Qt::AlignRight|Qt::AlignBottom, str);
+        }
     }
+    painter.setBackground(QBrush(m_color[0]));
 }
 
 
@@ -1013,7 +1078,6 @@ void Screen::bossColor()
 void Screen::drawMenuSelect(QPainter& painter, int index)
 {
     QRect rcSelect, rcMenu, rcInter;
-//  FIXME: what is this for
     if (m_pBuffer->isSelected(index)) {
         rcSelect = mapToRect(m_pBuffer->getSelectRect(index, m_pWindow->m_bCopyRect));
         if (Global::instance()->isBossColor())
@@ -1023,47 +1087,12 @@ void Screen::drawMenuSelect(QPainter& painter, int index)
     }
 
     if (m_pBBS->isSelected(index)) {
-        rcMenu = mapToRect(m_pBBS->getSelectRect());
-//   m_pBand->setGeometry(rcMenu);
-//   m_pBand->show();
+        rcMenu = mapToRect(m_pBBS->getSelectRect().intersected(QRect(0,index,m_pBuffer->columns(), 1)));
         switch (m_pParam->m_nMenuType) {
         case 0: // underline
-//     m_pBand->setGeometry(rcMenu.x(), rcMenu.y()+10*m_nCharHeight/11,
-//           rcMenu.width(), m_nCharHeight/11);
-//     m_pBand->show();
-//     pxm = new QPixmap( rcMenu.width(), m_nCharHeight/11);
-//     pxm->fill(m_color[7]);
-//     bitBlt(this, rcMenu.x(), rcMenu.y()+10*m_nCharHeight/11, pxm, 0, 0,
-//          rcMenu.width(), m_nCharHeight/11, Qt::NotROP);
-//     delete pxm;
             painter.fillRect(rcMenu.x(), rcMenu.y() + 10*m_nCharHeight / 11, rcMenu.width(), m_nCharHeight / 11, m_color[7]);
             break;
         case 2:
-//     rcInter = rcSelect.intersect(rcMenu);
-//     if(!rcInter.isEmpty())
-//     {
-//      pxm = new QPixmap(rcInter.width(), rcInter.height());
-//      pxm->fill(m_pParam->m_clrMenu);
-//      bitBlt(this, rcInter.x(), rcInter.y(), pxm, 0, 0,
-//         rcInter.width(), rcInter.height(), Qt::XorROP );
-//      delete pxm;
-//      QRect rcTmp;
-//      rcTmp.setBottom(rcMenu.bottom());
-//      rcTmp.setTop(rcMenu.top());
-//      if(rcMenu.left()<rcSelect.left())
-//      {
-//       rcTmp.setLeft(rcMenu.left());
-//       rcTmp.setRight(rcInter.left());
-//       painter.fillRect(rcTmp, QBrush(m_pParam->m_clrMenu));
-//      }
-//      if(rcMenu.right()>rcSelect.right())
-//      {
-//       rcTmp.setLeft(rcInter.right());
-//       rcTmp.setRight(rcMenu.right());
-//       painter.fillRect(rcTmp, QBrush(m_pParam->m_clrMenu));
-//      }
-//     }
-//     else
             painter.fillRect(rcMenu, QBrush(m_pParam->m_clrMenu));
             break;
         }
@@ -1107,9 +1136,9 @@ QRect Screen::mapToRect(int x, int y, int width, int height)
     QPoint pt = mapToPixel(QPoint(x, y));
 
     if (width == -1)  // to the end
-        return QRect(pt.x(), pt.y(), size().width() , m_nCharHeight*height);
+        return QRect(pt.x()+m_nCharDelta, pt.y(), size().width() , m_nCharHeight*height);
     else
-        return QRect(pt.x(), pt.y(), width*m_nCharWidth, m_nCharHeight*height);
+        return QRect(pt.x()+m_nCharDelta, pt.y(), width*m_nCharWidth, m_nCharHeight*height);
 }
 
 QRect Screen::mapToRect(const QRect& rect)
@@ -1187,8 +1216,32 @@ QImage& Screen::fade(QImage& img, float val, const QColor& color)
 
 void Screen::inputMethodEvent(QInputMethodEvent * e)
 {
-    QString text = e->commitString();
-    emit inputEvent(&text);
+    QString commitString = e->commitString();
+    if (!commitString.isEmpty()) {
+        emit inputEvent(commitString);
+    }
+    QString preeditString = e->preeditString();
+    if (m_inputContent == NULL){
+        m_inputContent = new Input(this, m_nCharWidth, m_nCharHeight, m_nCharAscent);
+        m_inputContent->setFont(*m_pGeneralFont);
+    }
+    if (preeditString.isEmpty()) {
+        m_inputContent->hide();
+    } else {
+        m_inputContent->drawInput(e);
+        QPoint cursor;
+        int x = m_pBuffer->caretX();
+        int y = m_pBuffer->caretY();
+        cursor = mapToPixel(QPoint(x+1,y));
+
+        if (m_inputContent->width() + cursor.x() > m_rcClient.width()){
+            cursor = mapToPixel(QPoint(x,y));
+            cursor.setX(m_rcClient.right() - m_inputContent->width());
+        }else
+            cursor = mapToPixel(QPoint(x,y));
+        m_inputContent->show();
+        m_inputContent->move(cursor);
+    }
 }
 
 QVariant Screen::inputMethodQuery(Qt::InputMethodQuery property) const
@@ -1199,140 +1252,82 @@ QVariant Screen::inputMethodQuery(Qt::InputMethodQuery property) const
         return QVariant(QRect((m_pBuffer->caret().x() + 1)*m_nCharWidth, (m_pBuffer->caret().y() + 1)*m_nCharHeight,
                               m_nCharWidth, m_nCharHeight));
     case Qt::ImFont:
-        return QVariant(*m_pFont);
+        return QVariant(*m_pGeneralFont);
     case Qt::ImCursorPosition:
         return m_pBuffer->caret().x();
     default:
         return QVariant();
     }
 }
-/*
-void Screen::imStartEvent(QIMEvent * e)
-{
- //m_inputContent = new QTermInput(this, m_nCharWidth, m_nCharHeight, m_nCharAscent);
- //m_inputContent->setFont(*m_pFont);
- //m_inputContent->setTextFormat(Qt::RichText);
 
- //m_inputContent->show();
+void Input::drawInput(QInputMethodEvent * e)
+{
+    int cursorPos = -1;
+    foreach (QInputMethodEvent::Attribute attribute, e->attributes()) {
+        switch (attribute.type) {
+            case QInputMethodEvent::TextFormat:
+                //FIXME: text format?
+                break;
+            case QInputMethodEvent::Cursor:
+                if (attribute.length == 0) {
+                    cursorPos = -1;
+                } else {
+                    cursorPos = attribute.start;
+                }
+                break;
+            case QInputMethodEvent::Language:
+                break;
+            case QInputMethodEvent::Ruby:
+                break;
+            default:
+                qDebug("Unknown attribute");
+                break;
+        }
+    }
+
+    d_text = e->preeditString();
+    d_pos = cursorPos;
+    repaint();
 }
 
-void Screen::imComposeEvent(QIMEvent * e)
+void Input::paintEvent(QPaintEvent * e)
 {
- if (m_inputContent == NULL){
-  m_inputContent = new QTermInput(this, m_nCharWidth, m_nCharHeight, m_nCharAscent);
-  m_inputContent->setFont(*m_pFont);
-  m_inputContent->show();
- }
- QString text = QString::null;
+    QPainter inputPainter;
+    int len = 0;
+    int cursor = 0;
+    int width, height;
 
- QPoint cursor;
- int x = m_pBuffer->caretX();
- int y = m_pBuffer->caretY();
- int pos = e->cursorPos();
+    if (d_pos == -1 || d_pos >= d_text.length())
+        d_pos = 0;
+    for (int i = 0; i < d_text.length(); ++i) {
+        if (i == d_pos)
+            cursor = len;
+        len += TermString::wcwidth(d_text[i]);
+    }
 
- text += e->text();
-// how dirty
-#ifdef Q_OS_MACX
-ushort code=text[text.length()-1].unicode();
-if( (code>0xff00&&code<0xffef) ||
-(code>0x3000&&code<0x303f) ||
-(code>0x2000&&code<0x206f) ||
-code==0x00b7)
-{
-emit inputEvent(&text);
-return;
+    width = len * d_width;
+    height = d_height;
+
+    inputPainter.begin(this);
+    setFixedSize(width,height);
+    inputPainter.eraseRect(rect());
+
+    len = 0;
+    inputPainter.setPen(Qt::black);
+
+    for (int j = 0; j < d_text.length(); ++j) {
+        if (j == d_pos) {
+            QRect rcCurrent(len * d_width, 0, d_width*TermString::wcwidth(d_text[d_pos]), d_height);
+            inputPainter.fillRect(rcCurrent, QBrush(Qt::darkGray));
+            inputPainter.setPen(Qt::white);
+            inputPainter.drawText(len * d_width ,d_ascent, d_text.mid(d_pos, 1));
+        } else {
+            inputPainter.drawText(len * d_width, d_ascent, d_text.mid(j, 1));
+        }
+        len += TermString::wcwidth(d_text[j]);
+    }
+    inputPainter.end();
 }
-#endif
- m_inputContent->drawInput(text, pos);
-
- QString text_before = text.left(pos);
- QString text_after = text.mid(pos + 1);
- QString text_select = "<u>" + text.mid(pos,1) + "</u>";
-
- text = text_before + text_select + text_after;
-
- cursor = mapToPixel(QPoint(x+1,y));
-
- //m_inputContent->setText(text);
- //m_inputContent->adjustSize();
-
- if (m_inputContent->width() + cursor.x() > m_rcClient.width()){
-  cursor = mapToPixel(QPoint(x,y));
-  cursor.setX(m_rcClient.right() - m_inputContent->width());
- }else
-  cursor = mapToPixel(QPoint(x,y));
- m_inputContent->move(cursor);
-}
-
-void Screen::imEndEvent(QIMEvent * e)
-{
- QString text = QString::null;
- text += e->text();
- //m_inputContent->setText(QString::null);
- delete m_inputContent;
- m_inputContent = NULL;
- emit inputEvent(&text);
-}
-*/
-/*
-void QTermInput::drawInput(QString & inputText, int position)
-{
- d_text = inputText;
- d_pos = position;
- repaint();
-}
-
-void QTermInput::paintEvent(QPaintEvent * e)
-{
- QPainter inputPainter;
- int len = 0;
- int cursor = 0;
- int width, height;
-
- if (d_pos == -1 || d_pos >= d_text.length())
-  d_pos = 0;
- for (int i = 0; i < d_text.length(); ++i) {
-  if (d_text[i] <= 0x7f) {
-   ++len;
-   if (i == d_pos)
-    cursor = len - 1;
-  }
-  else {
-   len += 2;
-   if (i == d_pos)
-    cursor = len - 2;
-  }
- }
-
- width = len * d_width;
- height = d_height;
-
- inputPainter.begin(this);
- setFixedSize(width,height);
- erase();
-
- len = 0;
- inputPainter.setPen(Qt::black);
- //inputPainter.drawText(0 ,m_nCharAscent, text);
-
- for (int j = 0; j < d_text.length(); ++j) {
-  inputPainter.drawText(len * d_width, d_ascent, d_text.mid(j, 1));
-  if (d_text[j] <= 0x7f)
-   ++len;
-  else
-   len += 2;
- }
-#ifndef Q_OS_MACX
- QRect rcCurrent(cursor * d_width, 0, d_width*2, d_height);
- if (d_text[d_pos] <= 0x7f)
-  rcCurrent.setRect(cursor * d_width, 0, d_width, d_height);
- erase(rcCurrent);
- inputPainter.fillRect(rcCurrent, QBrush(Qt::darkGray));
- inputPainter.setPen(Qt::white);
- inputPainter.drawText(cursor * d_width ,d_ascent, d_text.mid(d_pos, 1));
-#endif
- inputPainter.end();
-}*/
 
 } // namespace QTerm
 
