@@ -450,7 +450,7 @@ Zmodem::Zmodem(QObject *netinterface, int type)
 
 	sending = false;
 
-	transferstate = notransfer;
+	transferstate = NoTransfer;
 
 	zmodemTimer= new QTimer(this);
 	connect(zmodemTimer, SIGNAL(timeout()), this, SLOT(ZmodemTimeout()));
@@ -519,6 +519,17 @@ int Zmodem::ZmodemTInit(ZModem *info)
 
 	ZIFlush(info) ;
 
+	QString path = Global::instance()->fileCfg()->getItemValue("global","openfiledialog").toString();
+	if (strFileList.count() == 0)
+		strFileList = QFileDialog::getOpenFileNames(0, "Choose the files", path, "All files(*)");
+	if(strFileList.count()!=0)
+	{
+		QStringList::Iterator itFile = strFileList.begin();
+		QFileInfo fi(*itFile);
+		Global::instance()->fileCfg()->setItemValue("global","openfiledialog", fi.absoluteDir().absolutePath() );
+		Global::instance()->fileCfg()->save();	
+	}
+	
 	/* optional: send "rz\r" to remote end */
 	if( DoInitRZ ) {
 	  if( (err = ZXmitStr((uchar *)"rz\r", 3, info)) )
@@ -529,17 +540,8 @@ int Zmodem::ZmodemTInit(ZModem *info)
 	  return err ;
 
 	info->timeout = 60 ;
+	transferstate = WaitingForReply;  // transfer complete
 
-	QString path = Global::instance()->fileCfg()->getItemValue("global","openfiledialog").toString();
-	strFileList = QFileDialog::getOpenFileNames(0, "Choose the files", path, "All files(*)");
-	if(strFileList.count()!=0)
-	{
-		QStringList::Iterator itFile = strFileList.begin();
-		QFileInfo fi(*itFile);
-		Global::instance()->fileCfg()->setItemValue("global","openfiledialog", fi.absoluteDir().absolutePath() );
-		Global::instance()->fileCfg()->save();	
-	}
-	
 	zmodemlog("ZmodemTInit[%s]: sent ZRQINIT\n", sname(info)) ;
 
 	return 0 ;
@@ -622,7 +624,7 @@ int Zmodem::ZmodemAbort(ZModem *info)
 	ZIFlush(info) ;
 	ZOFlush(info) ;
 
-	transferstate = transferstop;  // transfer complete
+	transferstate = TransferStop;  // transfer complete
 	ZmodemReset(info);  //Tranfer complete, zmodem return to receive state
 
 	zmodemlog("ZmodemAbort[%s]: send CAN\n", sname(info));
@@ -986,6 +988,7 @@ int Zmodem::ZAttn(ZModem *info)
 
 void Zmodem::ZStatus(int type, int value, const char * status)
 {
+	QString msg = QString::fromLatin1(status);
 	emit ZmodemState(type, value, status);
 	switch(type)
 	{
@@ -1888,7 +1891,7 @@ int Zmodem::SendRinit(  ZModem *info )
 	  /* TODO: switch to Ymodem */
 #endif	/* COMMENT */
 
-	transferstate = transferstart; //transfer would be active, it must be set to false when transfer complete or abort
+	transferstate = TransferStart; //transfer would be active, it must be set to false when transfer complete or abort
 	zmodemlog("SendRinit[%s]: send ZRINIT\n", sname(info)) ;
 
 	info->timeout = ResponseTime ;
@@ -2183,7 +2186,7 @@ int Zmodem::GotFin(  ZModem *info )
 	  free(info->filename) ;
 	i= ZXmitHdrHex(ZFIN, zeros, info) ;
 	ZmodemReset( info);
-	transferstate = transferstop;  // transfer complete
+	transferstate = TransferStop;  // transfer complete
 	return i;
 }
 
@@ -2609,7 +2612,7 @@ int Zmodem::GotRinit(  ZModem *info )
 	 * ZCRCW: CRC next, send ZACK, frame ends, header follows
 	 */
 
-	transferstate = transferstart;
+	transferstate = TransferStart;
 
 	ZFlowControl(1,info) ;
 
@@ -2803,7 +2806,7 @@ int Zmodem::OverAndOut( ZModem *info )
 
 	ZXmitStr((uchar *)"OO", 2, info) ;
 
-	transferstate = transferstop;  // transfer complete
+	transferstate = TransferStop;  // transfer complete
 
 	ZmodemReset(info);  //Tranfer complete, zmodem return to receive state
 
@@ -2884,6 +2887,11 @@ void Zmodem::zmodemCancel()
 	ZmodemAbort(&info);
 }
 
+void Zmodem::setFileList(const QStringList & fileList)
+{
+	strFileList = fileList;
+}
+
 } // namespace QTerm
 
 #include <qtermzmodem.moc>
